# -*- coding: utf-8 -*-

"""
lassie.helpers
~~~~~~~~~~~~~~

This module contains the set of helper functions executed by Lassie methods.

"""

import locale
import re

from requests.utils import default_user_agent

from .compat import str, urljoin

CLEANER = re.compile(r'[\r\n\t]')
RE_INT = re.compile(r'\d+')
FAKE_USER_AGENT = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_4) AppleWebKit/603.1.20 (KHTML, like Gecko) Version/10.1 Safari/603.1.20'

def clean_text(value):
    """Removes all line breaks, new lines and tabs from the specified content

    :param value: Content to be cleansed
    :type value: string

    """
    return CLEANER.sub('', value)

def convert_to_int(value):
    """Attempts to convert a specified value to an integer

    :param value: Content to be converted into an integer
    :type value: string or int

    """
    if not value:
        return None

    # Apart from numbers also accept values that end with px
    if isinstance(value, str):
        value = value.strip(' px')

    try:
        return int(value)
    except (TypeError, ValueError):
        return None

def normalize_locale(value):
    value = value.replace('-', '_')
    the_locale = locale.normalize(value)

    if the_locale != value:
        # Should we return the actual locale, returned from the locale lib instead of splitting?
        try:
            return str(the_locale.split('.')[0])
        except IndexError:  # pragma: no cover
            pass
    return None

def normalize_image_data(data, url):
    # Create image list then remove duplicate images?
    img = {
        'src': urljoin(url, data.get('src')),
        'alt': data.get('alt', ''),
        'type': u'body_image',
    }

    # Only include width and height if included as an attribute of the element
    width = convert_to_int(data.get('width'))
    if width:
        img['width'] = width

    height = convert_to_int(data.get('height'))
    if height:
        img['height'] = height

    return img

def determine_user_agent(user_agent):
    if not user_agent or  user_agent == default_user_agent():
        return FAKE_USER_AGENT

    return user_agent



# -*- coding: utf-8 -*-

"""
lassie.filters.generic
~~~~~~~~~~~~~~~~~~~~~~

This module contains data about generic type content to help Lassie filter for content.

"""

import re

from ..compat import str

GENERIC_MAPS = {
    'meta': {
        'generic': {
            'pattern': re.compile(r"^(description|keywords|title)", re.I),
            'map': {
                'description': 'description',
                'keywords': 'keywords',
                'title': 'title',
            },
            'image_key': '',
            'video_key': '',
            'key': 'name',
        },
    },
    'link': {
        'favicon': {
            'pattern': 'icon',
            'key': 'rel',
            'type': str('favicon'),
        },
        'canonical': {
            'pattern': 'canonical',
            'key': 'rel',
            'type': 'url'
        }
    },
}



# -*- coding: utf-8 -*-

"""
lassie.filters.apple
~~~~~~~~~~~~~~~~~~~~

This module contains Apple related content to help Lassie filter for content.

"""

import re

from ..compat import str

APPLE_MAPS = {  # http://i.imgur.com/cla85xT.jpg
    'link': {
        'touch_icon': {
            'pattern': re.compile(r"^(apple-touch-icon|apple-touch-icon-precomposed)", re.I),
            'key': 'icon',
            'type': str('touch_icon'),
        },
    }
}



# -*- coding: utf-8 -*-

"""
lassie.filters.providers
~~~~~~~~~~

This module contains oembed providers and a python oembed consumer.

"""

import re

import oembed

from ...utils import convert_to_int

HYPERLINK_PATTERN = re.compile(r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+')

PROVIDERS = {
    'http://www.youtube.com/oembed': [
        'https?://*.youtube.com/watch*',
        'https?://*.youtube.com/v/*',
        'https?://youtu.be/*',
        'https?://*.youtube.com/user/*',
        'https?://*.youtube.com/*#*/*',
        'https?://m.youtube.com/index*',
        'https?://*.youtube.com/profile*',
        'https?://*.youtube.com/view_play_list*',
        'https?://*.youtube.com/playlist*'
    ]
}

consumer = oembed.OEmbedConsumer()
for k, v in PROVIDERS.items():
    endpoint = oembed.OEmbedEndpoint(k, v)
    consumer.addEndpoint(endpoint)


def parse_oembed_data(oembed_data, data):
    """Parse OEmbed resposne data to inject into lassie's response dict.

    :param oembed_data: OEmbed response data.
    :type oembed_data: dict
    :param data: Refrence to data variable being updated.
    :type data: dict

    """
    data.update({
        'oembed': oembed_data,
    })
    _type = oembed_data.get('type')
    provider_name = oembed_data.get('provider_name')
    if not _type:
        return data

    if oembed_data.get('title'):
        data.update({
            'title': oembed_data.get('title'),
        })

    if _type == 'video':
        try:
            item = {
                'width': convert_to_int(oembed_data.get('width')),
                'height': convert_to_int(oembed_data.get('height'))
            }
            if provider_name in ['YouTube', ]:
                item['src'] = HYPERLINK_PATTERN.search(oembed_data.get('html')).group(0)

            data['videos'].append(item)
        except Exception:
            pass

        if oembed_data.get('thumbnail_url'):
            item = {
                'width': convert_to_int(oembed_data.get('thumbnail_width')),
                'height': convert_to_int(oembed_data.get('thumbnail_height')),
                'src': oembed_data.get('thumbnail_url')
            }

            data['images'].append(item)

    return data



# -*- coding: utf-8 -*-

"""
lassie.filters
~~~~~~~~~~~~~~

This package contains various filters for parsing content.

"""

from .apple import APPLE_MAPS
from .generic import GENERIC_MAPS
from .social import SOCIAL_MAPS

META_MAPS = dict(list(GENERIC_MAPS['meta'].items()) + list(SOCIAL_MAPS['meta'].items()))
LINK_MAPS = dict(list(APPLE_MAPS['link'].items()) + list(GENERIC_MAPS['link'].items()))

FILTER_MAPS = {
    'meta': META_MAPS,
    'link': LINK_MAPS,
}
